#!/usr/bin/env ruby

require 'metasm'
include Metasm

# patch the control flow graph once dasm is finished
def fix_switch_static(dasm, sw_addr)
	# retrieve the address of the switch table
	mov = dasm.di_including(sw_addr-1)
	switch_addr = mov.instruction.args[1].imm.reduce

	# retrieve the content of the switch table
	switch_entries = []
	6.times { |nr|
		# retrieve each entry, push it into the switch_entries array
		switch_entries << dasm.decode_int(switch_addr + 4*nr, 4)
	}

	switch_entries.each { |addr|
		# now disassemble at each pointer and fix the disassembly graph
		dasm.disassemble(addr)
		dasm.di_at(sw_addr).block.to_normal << addr
		dasm.di_at(addr).block.from_normal ||= []
		dasm.di_at(addr).block.from_normal << sw_addr
	}
end

# load the target binary
file = 'grimcreeper.d'
e = AutoExe.decode_file(file)
d = e.disassembler

# disassemble from an entrypoint close to our target
sw_addr = 0x8048F2A
ep = 'parse_options'
d.disassemble(ep)

# do our stuff
fix_switch_static(d, sw_addr)

# run the gui to show the results
Gui::DasmWindow.new("metasm disassembler - #{file}", d, ep)
Gui.main

