#!/usr/bin/env ruby

require 'metasm'  # load metasm
include Metasm    # avoid Metasm::Bla

# patch the control flow graph once dasm is finished
def fix_switch_static(dasm, sw_addr)
	# get switch block
	sw_block = dasm.decoded[dasm.normalize(sw_addr)].block

	# get mov eax, [804a110h+4*edx] instruction from block's instructions list (next-to-last)
	di = sw_block.list[-2]

	# get jump table base from instruction's argument (which is a ModRm)
	modrm = di.instruction.args.last
	base = modrm.imm

	# get all switches entries
	switch_entries = []
	6.times { |idx|
		switch_entries << dasm.decode_dword( base + 4*idx )
	}

	# fix each switch entry
	switch_entries.each { |addr|
		puts "fixing #{Expression[addr]} entry"

		# disasm at entry
		dasm.disassemble(addr)

		# link blocks
		sw_block.add_to_normal addr						# switch -> block
		dasm.di_at(addr).block.add_from_normal(sw_block.list.last.address)	# block -> switch
		dasm.add_xref(addr, Xref.new(:x, sw_block.list.last.address))		# add a crossref
	}
end


file = 'grimcreeper.d'
sw_addr = 0x8048F2A
ep = 'parse_options'

e = AutoExe.decode_file(file)
d = e.disassembler

d.disassemble(ep)

fix_switch_static(d, sw_addr)

Gui::DasmWindow.new("metasm disassembler - #{file}", d, ep)
d.load_plugin 'hl_opcode'
Gui.main
