module Metasm
class VM_CPU < CPU      
	def initialize
		super()
		@endianness = :little
		@size = 32
	end

	REG_SYMS = [:eax, :ecx, :edx, :ebx, :esp, :ebp, :esi, :edi].reverse
	def register_symbols
		REG_SYMS
	end

	class Reg
		attr_accessor :i

		def ==(o)
			o.class == self.class and o.i == i
		end

		def initialize(i)
			@i = i
		end
		
		def symbolic
			REG_SYMS[@i]
		end

		include Renderable
		def render
			[REG_SYMS[@i].to_s]
		end
	end

	class Memref
		attr_accessor :base
		
		def initialize(base)
			@base = base
		end
		
		def symbolic(orig, sz)
			b = @base
			b = b.symbolic if b.kind_of? Reg
			Indirection[b, sz, orig]
		end

		include Renderable
		def render
			['[', @base,']']
		end
	end

	def build_bin_lookaside
	end

	def addop(name, bin, *args)
		o = Opcode.new(name)
		o.bin = bin
		(args & @valid_props.keys).each { |p| o.props[p] = true ; args.delete p }
		o.args = args
		@opcode_list << o
	end

	def init_opcode_list
		@opcode_list = []
		addop 'call', 0x75, :saveip, :stopexec, :imm
		addop 'mov', -0x69, :reg, :regimm
		addop 'push', 0x59, :regimm
		addop 'api_call',-0x18, :regimm
		addop 'add', 0x1d, :reg, :regimm
		addop 'cmp',-0x0a, :reg, :regimm
		addop 'jz',  0x13, :regimm, :setip
		addop 'jnz', 0x52, :regimm, :setip
		addop 'ret', 0x65, :setip, :stopexec
		addop 'mov', 0x33, :reg, :regimmptr
		addop 'and',-0x5f, :reg, :regimm
		addop 'jmp', 0x4b, :regimm, :setip, :stopexec
	end

	def decode_findopcode(edata)
		di = super(edata)
		bin = edata.decode_imm(:i8, :little)
		di.opcode = opcode_list.find { |op| op.bin == bin }
		if di.opcode
			di
		else
			puts "unknown instruction 0x#{bin.to_s 16}"
		end
	end

	def decode_instr_op(edata, di)
		argtype = edata.decode_imm(:i8, :little)
		di.bin_length = edata.decode_imm(:i8, :little) 
		#return if di.bin_length < 2 or di.bin_length > 8

		i = Instruction.new(self, di.opcode.name)
		di.instruction = i

		di.opcode.args.each { |a|
			case a
			when :imm
				i.args << Expression[edata.decode_imm(:u32, :little)]
			when :reg
				regnum = edata.decode_imm(:u8, :little)
				i.args << Reg.new(regnum/4)
			when :regimm
				if argtype == -0x52
					regnum = edata.decode_imm(:u8, :little)
					i.args << Reg.new(regnum/4)
				elsif argtype == -1
					i.args << Expression[edata.decode_imm(:u32, :little)]
				end
			when :regimmptr
				if argtype == -0x52
					regnum = edata.decode_imm(:u8, :little)
					i.args << Memref.new(Reg.new(regnum/4))
				elsif argtype == -1
					i.args << Memref.new(Expression[edata.decode_imm(:u32, :little)])
				end
			end
		}

		di
	end

	def get_backtrace_binding(di)
		args = di.instruction.args.map { |a|
			case a
			when Reg; a.symbolic
			when Memref; a.symbolic(di.address, 4)
			else a
			end
		}
		case di.opcode.name
		when 'call'; { :esp => Expression[:esp, :+, 4], Indirection[:esp, 4] => Expression[di.next_addr] }
		when 'ret'; { :esp => Expression[:esp, :-, 4] }
		when 'mov'; { args[0] => Expression[args[1]] }
		else
			{}
		end
	end

	def get_xrefs_x(dasm, di)
		case di.opcode.name
		when 'call', 'jz', 'jnz', 'jmp'
			di.instruction.args
		when 'ret'
			[Indirection[:esp, 4, di.address]]
		when 'api_call'
			di.instruction.args[0] = Expression[dasm.get_label_at(di.instruction.args[0])]
			[]
		else
			[]
		end
	end

	def backtrace_is_function_return(expr, di=nil)
		expr = Expression[expr].reduce_rec
		expr.kind_of? Indirection and expr.target == Expression[:esp]
	end

	def backtrace_update_function_binding(dasm, faddr, f, retaddrlist, *wantregs)
		retaddrlist.map! { |retaddr| dasm.decoded[retaddr] ? dasm.decoded[retaddr].block.list.last.address : retaddr } if retaddrlist
		b = f.backtrace_binding
		
		bt_val = lambda { |r|
			next if not retaddrlist
			bt = []
			b[r] = Expression::Unknown  # break recursive dep
			retaddrlist.each { |retaddr|
bt |= dasm.backtrace(Expression[r], retaddr,
	:include_start => true, :snapshot_addr => faddr, :origin => retaddr)
			}
		
			b[r] = ((bt.length == 1) ? bt.first : Expression::Unknown)
      		}
		
		wantregs = [:esp] if wantregs.empty?
		wantregs.map { |r| r.to_sym }.each(&bt_val)
	end

	def instr_uncond_jump_to(target)
		Instruction.new(self, 'jmp', [Expression[target]])
	end
end
end
