require 'metasm'
require File.expand_path('../vm_cpu', __FILE__)
include Metasm

ep = 0x40121D

pe = AutoExe.decode_file('crackme_nop_vm.exe')
pe.cpu = VM_CPU.new

d = pe.disassembler

labels = {
	0x4014E8 => 'thunk_ExitProcess',
	0x4014EE => 'thunk_GetStdHandle',
	0x4014F4 => 'thunk_ReadFile',
	0x4014FA => 'thunk_Sleep',
	0x401500 => 'thunk_WriteFile',
	0x403000 => 'pass_buffer',
}

labels.each { |a, n| d.set_label_at(a, n) }

d.disassemble(ep)

# with minor modifications, we can transform the VM bytecode into x86 asm and compile it in a new section in the PE !

# flatten the dasm instruction graph into a linear source file
src = d.flatten_graph(ep).join("\n")

# change VM-specific instruction into actual x86 instr
src = src.gsub('api_call', 'call')

# fix unresolved relocation
src << "\n" << 'loc_401238h: hlt'

puts src

# load a copy of the original PE file
pe2 = AutoExe.decode_file('crackme_nop_vm.exe')
# create a new section for the code, compile it in
pe2.parse '.section foobar rx'
pe2.parse src
pe2.assemble

# ensure calls to labels are correctly resolved
labels.each { |a, n|
	s = pe2.sect_at_va(a)
	s.encoded.add_export(n)
}

# encode the new PE file
# 'miny_copy' is needed to invalidate all the implicit fields of the PE headers read during #decode (nr of sections...)
pe2.mini_copy.encode_file 'nop2.exe'

puts "saved nop2.exe"
