module Metasm
class VM_CPU < CPU      
	def initialize
		super()
		@endianness = :little
		@size = 32
	end

	REG_SYMS = [:eax, :ecx, :edx, :ebx, :esp, :ebp, :esi, :edi].reverse
	def register_symbols
		REG_SYMS
	end

	class Reg
		attr_accessor :i

		def ==(o)
			o.class == self.class and o.i == i
		end

		def initialize(i)
			@i = i
		end
		
		def symbolic
			REG_SYMS[@i]
		end

		include Renderable
		def render
			[REG_SYMS[@i].to_s]
		end
	end

	class Memref
		attr_accessor :base
		
		def initialize(base)
			@base = base
		end
		
		def symbolic(orig, sz)
			b = @base
			b = b.symbolic if b.kind_of? Reg
			Indirection[b, sz, orig]
		end

		include Renderable
		def render
			['[', @base,']']
		end
	end

	def build_bin_lookaside
	end

	def addop(name, bin, *args)
		o = Opcode.new(name)
		o.bin = bin
		(args & @valid_props.keys).each { |p| o.props[p] = true ; args.delete p }
		o.args = args
		@opcode_list << o
	end

	def init_opcode_list
		@opcode_list = []
		addop 'call', 0x75, :imm, :saveip, :stopexec, :setip
	end

	def decode_findopcode(edata)
		di = DecodedInstruction.new(self)
		opcode = edata.decode_imm(:u8, @endianness)
		if op = opcode_list.find { |o| o.bin == opcode }
			di.opcode = op
			di
		else
			puts "Unhandled instruction #{opcode.to_s(16)}"
			nil
		end
	end

	def decode_instr_op(edata, di)
		di.instruction.opname = di.opcode.name
		case di.instruction.opname
		when 'call'
			edata.decode_imm(:u8, @endianness)
			edata.decode_imm(:u8, @endianness)
			di.instruction.args << Expression[edata.decode_imm(:u32, @endianness)]
			di.bin_length = 7
		end
		di
	end

	def get_backtrace_binding(di)
		{}
	end

	def get_xrefs_x(dasm, di)
		[]
	end

	def backtrace_is_function_return(expr, di=nil)
		Expression[expr].reduce == Expression[Indirection[:esp, 4]]
	end
end
end














