#!/usr/bin/ruby 

# extrait depuis les phases precedentes du challenge
smp = [0x00, 0xb0, 0x10, 0xd0, 0xb7, 0xa8, 0xa0, 0xb6, 0x0e, 0xb5, 0x71, 0xb4, 0x00, 0xbc, 0x01, 0xb6, 0x16, 0xb5, 0x71, 0xb4, 0x00, 0xbc, 0x52, 0xb6, 0xaf,
0xbe, 0x11, 0xb7, 0xa8, 0xb6, 0x24, 0xb5, 0x71, 0xb4, 0x00, 0xbc, 0xaf, 0xb1, 0xa9, 0xd0, 0xb6, 0x0f, 0x88, 0xd0, 0x96, 0xb6, 0xa9, 0xd0, 0xb7, 0x0f,
0x88, 0xd0, 0x8f, 0xb7, 0xaf, 0xa0, 0x8e, 0xb7, 0x40, 0xb6, 0x53, 0xb5, 0x00, 0xbd, 0xaf, 0xc1, 0x01, 0xb6, 0xa8, 0xb7, 0x4c, 0xb5, 0x71, 0xb4, 0x00,
0xbc, 0xaf, 0xb0, 0x02, 0xb6, 0x00, 0xbe, 0xc8, 0x01, 0xb5, 0x00, 0xb4, 0x6d, 0xb3, 0xad, 0xbb, 0x66, 0xb3, 0xac, 0xd8, 0xb4, 0xad, 0x8f, 0xbb, 0x01,
0x94, 0xb4, 0xad, 0xd8, 0xb5, 0x57, 0xb3, 0x00, 0xbb, 0xac, 0xb7, 0x00, 0xbe, 0x00, 0xb1, 0xb3, 0x01, 0xb2, 0x63, 0xe0, 0xb4, 0xaa, 0xbc, 0x2c, 0xe0,
0xb4, 0xaf, 0x8a, 0xbc, 0x16, 0xe0, 0xb4, 0xae, 0x8a, 0xbc, 0x0f, 0xe0, 0xb4, 0xa9, 0xbc, 0xab, 0x92, 0xb3, 0xaa, 0xb1, 0x59, 0xe0, 0xb4, 0x00, 0xbc,
0x22, 0xe0, 0xb4, 0xa9, 0xbc, 0xaa, 0xb1, 0x59, 0xe0, 0xb4, 0x00, 0xbc, 0xab, 0x92, 0xb3, 0x00, 0xb1, 0x59, 0xe0, 0xb4, 0x00, 0xbc, 0x48, 0xe0, 0xb4,
0xae, 0x8a, 0xbc, 0x3e, 0xe0, 0xb4, 0xa9, 0xbc, 0xaa, 0xb1, 0x59, 0xe0, 0xb4, 0x00, 0xbc, 0xaa, 0x93, 0xb3, 0x00, 0xb1, 0x59, 0xe0, 0xb4, 0x00, 0xbc,
0x57, 0xe0, 0xb4, 0xa9, 0xbc, 0xaa, 0x93, 0xb3, 0x00, 0xb1, 0x59, 0xe0, 0xb4, 0x00, 0xbc, 0x00, 0xb1, 0xaa, 0xd8, 0xb2, 0xa9, 0xd8, 0xb1, 0x76, 0xb4,
0x00, 0xbc, 0xab, 0xb7, 0x00, 0xbd,]

require 'metasm'

class SSTIC13_CPU < Metasm::CPU
	class Reg
		include Metasm::Renderable

		def initialize(i)
			@i = i
		end

		def symbolic
			"r#@i".to_sym
		end

		def render
			["r#@i"]
		end
	end

	class Acc
		include Metasm::Renderable

		def symbolic
			:acc
		end

		def render
			['acc']
		end
	end

	class MemRef
		include Metasm::Renderable

		def initialize(ptr)
			@ptr = ptr
		end

		def symbolic
			p = @ptr
			p = @ptr.symbolic if not @ptr.kind_of?(Metasm::Expression)
			Metasm::Indirection[p, 1]
		end

		def render
			['[', @ptr, ']']
		end
	end

	def initialize
		@size = 8
		@endianness = :little
	end

	def addop(name, bin, *args)
		o = Metasm::Opcode.new(name, bin)
		args.each { |a|
			o.args << a if @valid_args[a]
			o.props[a] = true if @valid_props[a]
			o.fields[a] = [@fields_mask[a], @fields_shift[a]] if @fields_mask[a]
		}
		@opcode_list << o
	end

	def init_opcode_list
		@opcode_list = []

		@valid_props = { :setip => true, :stopexec => true }
		@fields_mask = { :r => 7, :i7 => 0x7f, :m_r => 7 }
		@fields_shift = { :r => 0, :i7 => 0, :m_r => 0 }
		@valid_args = { :r => true, :i7 => true, :acc => true,
			:m_acc => true, :m_r => true, :i80 => true }

		# 00 mov acc, i7 	 # 0a mov acc, 0xa
		# 88 and acc, reg
		# 90 or  acc, reg
		# a0 not acc
		# a8 mov acc, reg
		# b0 mov reg, acc
		# b8 jz reg
		# c0 mov [reg], acc
		# c8 end
		# d0 mov acc, [acc]
		# d8 nop ?
		# e0 nop ?
		
		addop 'mov', 0x00, :acc, :i7
	end
	
	def build_bin_lookaside
		opcode_list.each { |op|
			mask = 0
			op.fields.each { |fld, (msk, shft)| mask |= msk << shft }
			op.bin_mask = ~mask
		}
	end

	def decode_findopcode(edata)
		di = Metasm::DecodedInstruction.new(self)
		val = edata.decode_imm(:u8, @endianness)
		edata.ptr -= 1
		if op = opcode_list.find { |o| val & o.bin_mask == o.bin }
			di.opcode = op
			di
		end
	end

	def decode_instr_op(edata, di)
		val = edata.decode_imm(:u8, @endianness)

		di.instruction.opname = di.opcode.name

		di.opcode.args.each { |a|
			instr_arg = case a
			when :acc
				Acc.new
			when :i7
				fldval = (val >> @fields_shift[a]) & @fields_mask[a]
				Metasm::Expression[fldval]
			else raise "unhandled argument #{a.inspect}"
			end
			di.instruction.args << instr_arg
		}

		di.bin_length = 1

		di
	end

	def get_backtrace_binding(di)
		args = di.instruction.args.map { |a|
			case a
			when Reg, Acc, MemRef; a.symbolic
			when Metasm::Expression; a
			end
		}

		case di.opcode.name
		when 'mov'; { args[0] => Metasm::Expression[args[1]] }
		else puts "Unhandled backtrace #{di}" ; {}
		end
	end

	def get_xrefs_x(dasm, di)
		[]
	end
end

# convert byte array to String
bin = smp.pack('C*')

# wrap in EncodedData
edata = Metasm::EncodedData.new(bin)

# init a Shellcode disassembler
dasm = Metasm::Shellcode.decode(edata, SSTIC13_CPU.new).disassembler

dasm.disassemble(0)

#puts dasm

Metasm::Gui::DasmWindow.new("sstic2013", dasm)
dasm.load_plugin 'hl_opcode'
Metasm::Gui.main
